extends Sprite

const GHOST_EFFECT = preload("res://GhostEffect.tscn")

const SPEED := 500.0
const JUMP_POWER := 1000.0
const GRAVITY := 3000.0
const JUMP_SCALE_TIME := 0.2

# 状態
enum eState {
	IDLE,
	RUN, # 走る
	JUMP # ジャンプ
}

# ジャンプによるスケール状態
enum eJumpScale {
	NONE,
	JUMPING, #ジャンプ開始
	LANDING, # 着地開始
}

# 移動量
var velocity = Vector2()

var anim_time = 0
var state = eState.IDLE
var jump_scale = eJumpScale.NONE
var jump_scale_timer = 0
var is_left = false # 左を向いているかどうか
var ghost_cnt = 0

# ゴーストエフェクトを表示する CanvasLayer
onready var _ghost_effects = $"../GhostEffectLayer"

func _ready() -> void:
	pass # Replace with function body.

func _process(delta: float) -> void:
	
	# ジャンプ判定
	_update_jump()
	
	# 横移動
	_move_horizontal(delta)
				
	# 重力と着地を処理する
	_update_landing(delta)
	
	# 残像エフェクトの処理
	_proc_ghost_effect()
	
	# アニメーションタイマー更新
	anim_time += delta
	# スプライトフレームを設定
	frame = _get_anim_frame()
	flip_h = is_left # 反転する
	
	# ジャンプ・着地演出
	_update_jump_scale_anim(delta)

# ジャンプ判定
func _update_jump() -> void:
	if state == eState.JUMP:
		return # ジャンプ中はジャンプできない

	# Spaceキーでジャンプ開始
	if Input.is_action_just_pressed("ui_accept"):
		velocity.y = -JUMP_POWER
		state = eState.JUMP
		jump_scale = eJumpScale.JUMPING
		jump_scale_timer = JUMP_SCALE_TIME

# 左右移動判定
func _move_horizontal(delta:float) -> void:
	# 横移動の判定
	velocity.x = 0
	if Input.is_action_pressed("ui_left"):
		velocity.x = -1 # 左に移動
		is_left = true
	elif Input.is_action_pressed("ui_right"):
		velocity.x = 1 # 右に移動
		is_left = false

	if state != eState.JUMP:
		# ジャンプ中でなければ走り判定
		if velocity.x == 0:
			state = eState.IDLE
		else:
			state = eState.RUN
	position.x += velocity.x * delta * SPEED

# 重力と着地を処理する
func _update_landing(delta:float) -> void:
	velocity.y += GRAVITY * delta
	position.y += velocity.y * delta
	if position.y > 440:
		# 床に着地
		position.y = 440
		velocity.y = 0
		if state == eState.JUMP:
			# 着地した直後
			state = eState.RUN
			jump_scale = eJumpScale.LANDING
			jump_scale_timer = JUMP_SCALE_TIME

# 残像エフェクトの処理	
func _proc_ghost_effect() -> void:
	if state == eState.IDLE:
		ghost_cnt = 0
		return # 停止中は何もしない
		
	# 残像エフェクトを生成判定
	ghost_cnt += 1
	if ghost_cnt%6== 2:
		# 残像エフェクト生成
		var eft = GHOST_EFFECT.instance()
		eft.start(position, scale, frame, flip_h)
		_ghost_effects.add_child(eft)
	

# 現在の状態に対するアニメーションフレーム番号を取得する	
func _get_anim_frame() -> int:
	match state:
		eState.IDLE:
			var t = int(anim_time * 8) % 10
			if t == 0:
				t = 1
			else:
				t = 0
			return 0 + t
		eState.RUN:
			var t = int(anim_time * 8) % 2
			return 2 + t
		_:
			return 2

# ジャンプ・着地によるスケールアニメーションの更新
func _update_jump_scale_anim(delta:float) -> void:
	jump_scale_timer -= delta
	if jump_scale_timer <= 0:
		# 演出終了
		jump_scale = eJumpScale.NONE
	match jump_scale:
		eJumpScale.JUMPING:
			# 縦に伸ばす
			var d = 0.4 * jump_scale_timer / JUMP_SCALE_TIME
			scale.x = 1 - d
			scale.y = 1 + d
		eJumpScale.LANDING:
			# 縦に潰す
			var d = 0.4 * jump_scale_timer / JUMP_SCALE_TIME
			scale.x = 1 + d
			scale.y = 1 - d
		_:
			# もとに戻す
			scale.x = 1
			scale.y = 1
